@@ -9,7 +9,6 @@ use Carp         ();
 use Text::Xslate::Parser;
 use Text::Xslate::Util qw(
     $DEBUG
-    literal_to_value
     value_to_literal
     is_int any_in
     make_error
@@ -103,15 +102,27 @@ my %goto_family = map { $_ => undef } qw(
 );
 
 my %builtin = (
-    'raw'          => 'builtin_mark_raw',
-    'html'         => 'builtin_html_escape',
+    'html_escape'  => ['builtin_html_escape',
+                        \&Text::Xslate::Util::html_escape],
+    'uri_escape'   => ['builtin_uri_escape',
+                        \&Text::Xslate::Util::uri_escape],
+    'mark_raw'     => ['builtin_mark_raw',
+                        \&Text::Xslate::Util::mark_raw],
+    'unmark_raw'   => ['builtin_unmark_raw',
+                        \&Text::Xslate::Util::unmark_raw],
 
-    'mark_raw'     => 'builtin_mark_raw',
-    'unmark_raw'   => 'builtin_unmark_raw',
+    'raw'          => ['builtin_mark_raw',
+                        \&Text::Xslate::Util::mark_raw],
 
-    'uri'          => 'builtin_uri',
-    'is_array_ref' => 'builtin_is_array_ref',
-    'is_hash_ref'  => 'builtin_is_hash_ref',
+    'html'         => ['builtin_html_escape',
+                        \&Text::Xslate::Util::html_escape],
+    'uri'          => ['builtin_uri_escape',
+                        \&Text::Xslate::Util::uri_escape],
+
+    'is_array_ref' => ['builtin_is_array_ref',
+                        \&Text::Xslate::Util::is_array_ref],
+    'is_hash_ref'  => ['builtin_is_hash_ref',
+                        \&Text::Xslate::Util::is_hash_ref],
 );
 
 has lvar_id => ( # local varialbe id
@@ -145,6 +156,7 @@ has macro_table => (
 
 has engine => ( # Xslate engine
     is       => 'ro',
+    isa      => 'Object',
     required => 0,
     weak_ref => 1,
 );
@@ -231,6 +243,13 @@ has file => (
     init_arg => undef,
 );
 
+has overridden_builtin => (
+    is  => 'ro',
+    isa => 'HashRef',
+
+    default => sub { +{} },
+);
+
 sub lvar_use {
     my($self, $n) = @_;
 
@@ -259,6 +278,16 @@ sub compile {
     local $self->{current_file} = '<string>'; # for opinfo
     local $self->{file}         = $args{file} || \$input;
 
+    if(my $engine = $self->engine) {
+        my $ob = $self->overridden_builtin;
+        Internals::SvREADONLY($ob, 0);
+        while(my $name = each %builtin) {
+            my $f = $engine->{function}{$name};
+            $ob->{$name} = ( $builtin{$name}[1] != $f ) + 0;
+        }
+        Internals::SvREADONLY($ob, 1);
+    }
+
     my $parser = $self->parser;
 
     my $header = delete $self->{header};
@@ -471,7 +500,7 @@ sub _process_cascade {
 
 sub _process_cascade_file {
     my($self, $file, $base_code) = @_;
-    printf STDERR "cascade file %s\n", Mouse::Util::dump($file) if _DUMP_CAS;
+    printf STDERR "cascade file %s\n", p($file) if _DUMP_CAS;
     my $mtable = $self->macro_table;
 
     for(my $i = 0; $i < @{$base_code}; $i++) {
@@ -586,7 +615,7 @@ sub _generate_operator {
     $self->_error("Invalid expression", $node);
 }
 
-sub _can_print_optimize {
+sub _can_optimize_print {
     my($self, $name, $node) = @_;
 
     return 0 if !$OPTIMIZE;
@@ -598,6 +627,7 @@ sub _can_print_optimize {
         && any_in($node->first->id, qw(raw mark_raw html));
 }
 
+# also deal with smart escaping
 sub _generate_print {
     my($self, $node) = @_;
 
@@ -609,20 +639,34 @@ sub _generate_print {
     }
 
     foreach my $arg(@{ $node->first }){
-        if(exists $Text::Xslate::OPS{$proc . '_s'} && $arg->arity eq 'literal'){
+        if( $proc eq 'print' && $self->overridden_builtin->{html_escape} ) {
+            push @code,
+                $self->opcode('pushmark'),
+                $self->compile_ast($arg),
+                $self->opcode('push'),
+                $self->opcode('fetch_symbol' => 'html_escape'),
+                $self->opcode('funcall'),
+                $self->opcode('print_raw');
+        }
+        elsif(exists $Text::Xslate::OPS{$proc . '_s'}
+                && $arg->arity eq 'literal'){
             push @code,
                 $self->opcode( $proc . '_s' => $arg->value,
                                line         => $arg->line );
         }
-        elsif($self->_can_print_optimize($proc, $arg)){
+        elsif($self->_can_optimize_print($proc, $arg)){
             my $filter      = $arg->first;
             my $filter_name = $filter->id;
-            my $command = $builtin{ $filter_name } eq 'builtin_mark_raw'
+            my $command = $builtin{ $filter_name }[0] eq 'builtin_mark_raw'
                 ? 'print_raw'  # mark_raw, raw
                 : 'print';     # html
+
             push @code,
                 $self->compile_ast($arg->second->[0]),
-                $self->opcode( $command => undef, symbol => $filter );
+                $self->opcode(
+                    $command => undef,
+                    symbol   => $filter );
+
         }
         else {
             push @code,
@@ -1191,7 +1235,11 @@ sub _generate_call {
         if(@{$args} != 1) {
             $self->_error("Wrong number of arguments for $callable", $node);
         }
-        return $self->compile_ast($args->[0]), [ $intern => undef, $node->line ];
+
+        if( !$self->overridden_builtin->{ $intern->[0] } ) {
+            return $self->compile_ast($args->[0]),
+                [ $intern->[0] => undef, $node->line ];
+        }
     }
 
     return(
